# src/timesl/__init__.py

from .__version__ import __version__
from .core.sleep import sleep
from .clock import now, format_time

__all__ = [
    "sleep",
    "now",
    "format_time",
    "__version__",
]


# src/timesl/core/state.py

class VirtualTimeState:
    def __init__(self):
        self.active = False
        self.offset_ms = 0

    def advance(self, ms: int):
        self.offset_ms += ms


VT_STATE = VirtualTimeState()


# src/timesl/core/sleep.py

import time
from ..units import UNITS, DEFAULT_UNIT
from ..config import VT_THRESHOLD_UNIT, ENABLE_VT
from ..exceptions import InvalidUnitError
from .vt import vt_sleep

def sleep(value: float, unit: str = DEFAULT_UNIT):
    if unit not in UNITS:
        raise InvalidUnitError(f"Unknown time unit: {unit}")

    factor = UNITS[unit]

    # Chuyển VT
    if factor == "VT":
        if ENABLE_VT:
            vt_sleep(value)
            return
        else:
            raise RuntimeError("Virtual Time disabled")

    ms = int(value * factor)
    time.sleep(ms / 1000)


# tests/test_sleep.py

import timesl as ts
import time

def test_sleep_milliseconds():
    start = time.time()
    ts.sleep(100)  # 100 ms
    end = time.time()

    assert 0.09 <= (end - start) <= 0.2


def test_sleep_seconds():
    start = time.time()
    ts.sleep(1, "s")
    end = time.time()

    assert 0.9 <= (end - start) <= 1.2


def test_invalid_unit():
    try:
        ts.sleep(1, "abc")
        assert False
    except Exception:
        assert True


# src/timesl/calendarx.py

import calendar

def is_leap(year: int) -> bool:
    return calendar.isleap(year)

def days_in_month(year: int, month: int) -> int:
    return calendar.monthrange(year, month)[1]


# src/timesl/units.py

# Tất cả quy đổi về mili giây
UNITS = {
    # nhỏ
    "nns": 1e-6,        # nanosecond
    "mcs": 1e-3,        # microsecond
    "mls": 1,           # millisecond (mặc định)

    # lớn
    "s":   1000,
    "mn":  60_000,
    "h":   3_600_000,
    "d":   86_400_000,

    # rất lớn (chuyển VT)
    "m":   "VT",   # tháng
    "y":   "VT",   # năm
    "dc":  "VT",   # thập kỷ
    "ct":  "VT",   # thế kỷ
    "mlm": "VT",   # thiên niên kỷ
}

DEFAULT_UNIT = "mls"


# src/timesl/config.py

# Đơn vị mặc định: mili giây
DEFAULT_UNIT = "mls"

# Từ đơn vị này trở lên → chuyển sang Virtual Time
VT_THRESHOLD_UNIT = "d"   # ngày

# Độ chính xác tối đa (nanosecond)
MAX_PRECISION = "nns"

# Cho phép VT hay không
ENABLE_VT = True


# src/timesl/exceptions.py

class TimeSLException(Exception):
    """Base exception for timesl"""
    pass


class InvalidUnitError(TimeSLException):
    pass


class VirtualTimeRequired(TimeSLException):
    pass